import { FlatList, Text, TouchableOpacity, View } from "react-native";
import Card from "../../Components/Card";
import {
  AppContext,
  AppSelectedUserDevice,
  AppStyles,
  Constants,
  GetDevice,
} from "../../utils";
import { useCallback, useContext, useState } from "react";
import MyDropdown from "../../Components/Dropdown";
import MyIcon from "../../Components/Icon";
import { useTranslation } from "react-i18next";
import MyResult from "../../Components/Result";
import { MyIconButton, MyTextButton } from "../../Components/Button";
import { useFocusEffect } from "@react-navigation/native";
import { Divider } from "../../Components/Divider";
import { MyColorSwatch } from "../../Components/ColorPicker";
import { MyDotsModal } from "../../Components/Modal";

export default function SceneView({ navigation }) {
  const appContext = useContext(AppContext);
  const { t } = useTranslation();

  const [device, setDevice] = useState();
  const [isDotsModalOpen, setIsDotsModalOpen] = useState(false);

  const getSelectedDeviceSceneName = () => {
    let scene;

    if (device.selectedScene !== "") {
      scene = appContext.deviceScenes.find(
        (s) => s.sceneId === device.selectedScene
      );
    }

    return scene === undefined
      ? t("screens.device.scenes.dropdownSceneSelection.noSceneSelected")
      : scene.name;
  };

  useFocusEffect(
    useCallback(() => {
      setDevice(GetDevice(appContext.devices));

      if (device !== undefined) {
        if (device.selectedScene !== null) {
          navigation.setOptions({
            headerRight: () => (
              <MyIconButton
                iconName="dots-vertical"
                iconSize={24}
                style={AppStyles.headerNavigationIcons}
                onPress={() => setIsDotsModalOpen(true)}
              />
            ),
          });
        }
      }
    }, [device])
  );

  if (device === undefined) return <></>;

  const getActionTypeIcon = (actionType) => {
    let iconName = "";

    switch (actionType) {
      case Constants.actionType.layers:
        iconName = "lightbulb-on-outline";
        break;
      case Constants.actionType.ambilight:
        iconName = "television-ambient-light";
        break;
      case Constants.actionType.motor:
        iconName = "axis-z-rotate-counterclockwise";
        break;
      default:
        iconName = "help";
        break;
    }

    return <MyIcon name={iconName} size={24} />;
  };

  const deviceSceneActions = appContext.deviceSceneActions.filter(
    (a) => a.sceneId === device.selectedScene
  );

  return (
    <>
      <Card>
        <MyDropdown
          label={t("screens.device.scenes.dropdownSceneSelection.label")}
          onPress={() =>
            navigation.navigate("modalChooseScene", {
              device: { selectedScene: device.selectedScene },
            })
          }
          selectedItemLabel={getSelectedDeviceSceneName()}
        />
      </Card>

      {device.selectedScene === "" ? (
        <MyResult
          text={t("screens.device.scenes.infoNoSceneSelected")}
          iconName={"selection-search"}
        />
      ) : (
        <View>
          <FlatList
            scrollEnabled={false}
            data={deviceSceneActions}
            keyExtractor={(item) => item.actionId}
            ListEmptyComponent={
              <MyResult
                text={t("screens.device.scenes.infoNoActionsAvailableInScene")}
                iconName={"selection-search"}
              />
            }
            ListFooterComponent={
              <MyTextButton
                title={t("screens.device.scenes.buttonAddAction")}
                styleContainer={{
                  alignItems: "center",
                  marginTop: 10,
                }}
                style={{ padding: 8 }}
                actionColor={
                  deviceSceneActions === undefined ||
                  deviceSceneActions.length === 0
                }
                onPress={() =>
                  navigation.navigate("modalAddSceneAction", {
                    deviceFirmwareVersion: device.firmware.version,
                  })
                }
                iconName="plus-circle-outline"
              />
            }
            renderItem={({ item }) => {
              console.log("item", item);

              return (
                <View style={{ marginLeft: 10, marginRight: 10 }}>
                  <TouchableOpacity
                    onPress={() =>
                      navigation.navigate("modalLayersEditAction", {
                        actionId: item.actionId,
                        deviceFirmwareVersion: device.firmware.version,
                      })
                    }
                  >
                    <View
                      style={{
                        flexDirection: "row",
                        alignItems: "center",
                        justifyContent: "space-between",
                      }}
                    >
                      <View
                        style={{ flexDirection: "row", alignItems: "center" }}
                      >
                        {getActionTypeIcon(item.type)}
                        <View style={{ marginLeft: 12 }}>
                          <Text
                            style={{
                              color: appContext.appTheme.text,
                            }}
                          >
                            Rainbow
                          </Text>

                          <View
                            style={{
                              flexDirection: "row",
                              alignItems: "center",
                            }}
                          >
                            <Text
                              style={{
                                color: appContext.appTheme.textSecondary,
                              }}
                            >
                              Set to
                            </Text>

                            <MyColorSwatch
                              style={{ marginLeft: 6 }}
                              size={16}
                              backgroundColor={"red"}
                            />
                            <MyColorSwatch
                              size={16}
                              backgroundColor={"orange"}
                            />
                            <MyColorSwatch size={16} backgroundColor={"blue"} />
                          </View>
                        </View>
                      </View>

                      <MyIcon name="menu" size={20} />
                    </View>

                    <Divider />
                  </TouchableOpacity>
                </View>
              );
            }}
          />
        </View>
      )}

      <MyDotsModal
        isOpen={isDotsModalOpen}
        closeModal={() => setIsDotsModalOpen(false)}
        data={[
          {
            icon: "pencil",
            label: "Change scene name",
            onPress: () =>
              navigation.navigate("modalUpdateSceneName", {
                deviceSelectedScene: device.selectedScene,
                sceneName: getSelectedDeviceSceneName(),
              }),
          },
          {
            icon: "trash-can",
            label: "Delete scene",
            disabled: GetDevice(appContext.devices).selectedScene === "",
            onPress: () => {
              appContext.setDeviceScenes((scenes) =>
                scenes.filter(
                  (scene) =>
                    scene.sceneId !==
                    GetDevice(appContext.devices).selectedScene
                )
              );

              appContext.setDevices((arr) => {
                const newArr = [...arr];

                const foundDeviceIndex = arr.findIndex(
                  (d) => d.id === AppSelectedUserDevice.current.id
                );

                if (foundDeviceIndex !== -1) {
                  newArr[foundDeviceIndex].selectedScene = "";

                  appContext.setDeviceSceneActions((actions) =>
                    actions.filter(
                      (action) =>
                        action.sceneId !==
                        newArr[foundDeviceIndex].selectedScene
                    )
                  );
                }

                return newArr;
              });
            },
          },
        ]}
      />
    </>
  );
}
